package core.api.controller;


import core.api.models.auth.SimpleStatusResponse;
import core.api.models.timemanager.taskSchedule.*;
import core.entities.timemanager.AbstractSchedule;
import core.entities.timemanager.BasicTaskSchedule;
import core.entities.timemanager.Task;
import core.services.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.time.LocalDate;
import java.util.List;


@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/api")
public class ScheduleController {

    @Autowired private TaskScheduleService taskScheduleService;
    @Autowired private TaskService taskService;
    @GetMapping("/schedules")
    public ResponseEntity<?> loadAllSchedulesOfUser() {
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        List<AbstractSchedule> schedules = taskScheduleService.getAllSchedulesOfUser(username);

        return ResponseEntity.ok(schedules.stream().map(AbstractSchedule::toScheduleInfo).toList());
    }

    @GetMapping("/schedules/{taskID}")
    public ResponseEntity<?> loadAllSchedulesOfTask(@PathVariable long taskID) {
        PermissionResult<Task> permissionResult = taskService.getTaskPermissions(taskID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(permissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(permissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        List<AbstractSchedule> taskSchedules = permissionResult.getResult().getBasicTaskSchedules();
        return ResponseEntity.ok(taskSchedules.stream().map(AbstractSchedule::toScheduleInfo).toList());
    }

    @PutMapping("/schedules/{taskID}")
    public ResponseEntity<?> createSchedule(@PathVariable long taskID, @RequestBody @Valid ScheduleFieldInfo scheduleFieldInfo) {
        PermissionResult<Task> permissionResult = taskService.getTaskPermissions(taskID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(permissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(permissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        if(scheduleFieldInfo instanceof BasicScheduleFieldInfo) {
            ServiceResult<AbstractSchedule> scheduleResult = taskScheduleService.scheduleBasic(permissionResult.getResult(), (BasicScheduleFieldInfo) scheduleFieldInfo);
            return ResponseEntity.ok(scheduleResult.getResult().toScheduleInfo());
        } else {
            return ResponseEntity.status(400).body(new SimpleStatusResponse("failed"));
        }
    }

    @PostMapping("/schedules/{scheduleID}")
    public ResponseEntity<?> editSchedule(@PathVariable long scheduleID, @RequestBody @Valid ScheduleFieldInfo scheduleFieldInfo) {
        PermissionResult<AbstractSchedule> permissionResult = taskScheduleService.getSchedulePermissions(scheduleID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(permissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(permissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        if(permissionResult.getResult() instanceof BasicTaskSchedule && scheduleFieldInfo instanceof BasicScheduleFieldInfo) {
            ServiceResult<AbstractSchedule> updatedSchedule = taskScheduleService.editBasicSchedule((BasicTaskSchedule) permissionResult.getResult(), (BasicScheduleFieldInfo) scheduleFieldInfo);
            if(updatedSchedule.getExitCode() == ServiceExitCode.INVALID_OPERATION) {
                return ResponseEntity.status(400).body(new SimpleStatusResponse("failed"));
            } else {
                return ResponseEntity.ok(updatedSchedule.getResult().toScheduleInfo());
            }
        }

        return ResponseEntity.status(400).body(new SimpleStatusResponse("failed"));
    }

    @DeleteMapping("/schedules/{scheduleID}")
    public ResponseEntity<?> deleteSchedule(@PathVariable long scheduleID) {
        PermissionResult<AbstractSchedule> permissionResult = taskScheduleService.getSchedulePermissions(scheduleID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(permissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(permissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        taskScheduleService.deleteSchedule(permissionResult.getResult());
        return ResponseEntity.ok(new SimpleStatusResponse("success"));
    }

    @GetMapping("/schedules/{date}/{startable}")
    public ResponseEntity<?> loadFilteredSchedulesOfUser(@PathVariable String date, @PathVariable boolean startable) {
        List<AbstractSchedule> abstractSchedules = taskScheduleService.getFilteredScheduledOfUser(LocalDate.parse(date), startable, SecurityContextHolder.getContext().getAuthentication().getName());

        return ResponseEntity.ok(abstractSchedules.stream().map(AbstractSchedule::toScheduleInfo).toList());
    }

    @PostMapping("/schedules/{taskID}/now")
    public ResponseEntity<?> scheduleNow(@PathVariable long taskID) {
        PermissionResult<Task> permissionResult = taskService.getTaskPermissions(taskID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(permissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(permissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        ServiceResult<AbstractSchedule> scheduleResult = taskScheduleService.scheduleNow(permissionResult.getResult());
        if(scheduleResult.getExitCode() == ServiceExitCode.ENTITY_ALREADY_EXIST) {
            return ResponseEntity.status(409).body(new SimpleStatusResponse("failed"));
        } else {
            return ResponseEntity.ok(scheduleResult.getResult().toScheduleInfo());
        }
    }

    @GetMapping("/schedules/active")
    public ResponseEntity<?> getActiveSchedule() {
        ServiceResult<AbstractSchedule> serviceResult = taskScheduleService.getActiveSchedule(SecurityContextHolder.getContext().getAuthentication().getName());
        if(serviceResult.getExitCode() == ServiceExitCode.OK) {
            return ResponseEntity.ok(serviceResult.getResult().toScheduleInfo());
        } else {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }
    }

    @PostMapping("schedules/{scheduleID}/activate")
    public ResponseEntity<?> activateSchedule(@PathVariable long scheduleID) {
        PermissionResult<AbstractSchedule> permissionResult = taskScheduleService.getSchedulePermissions(scheduleID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(permissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(permissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        ServiceResult<AbstractSchedule> serviceResult = taskScheduleService.activateSchedule(permissionResult.getResult());
        return ResponseEntity.ok(new ScheduleActivateResponse(serviceResult.getResult().getStartTime()));
    }

    @PostMapping("schedules/{scheduleID}/stop/{finish}")
    public ResponseEntity<?> stopSchedule(@PathVariable long scheduleID, @PathVariable boolean finish) {
        PermissionResult<AbstractSchedule> permissionResult = taskScheduleService.getSchedulePermissions(scheduleID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(permissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(permissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        ServiceResult<AbstractSchedule> serviceResult = taskScheduleService.stopSchedule(permissionResult.getResult(), finish);
        return ResponseEntity.ok(new TaskScheduleStopResponse(serviceResult.getResult().getActiveTime()));
    }
}
